package qsl.learn;

import android.content.Context;
import android.graphics.PixelFormat;
import android.graphics.Point;
import android.hardware.Camera;
import android.hardware.Camera.Parameters;
import android.hardware.Camera.Size;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceHolder.Callback;
import android.view.SurfaceView;

/**
 * 这个类是拍照时的预览容器类，他接受了页面的camera，并且设置了camera 的属性，并开始预览。并且继承了callback回调接口
 * 
 * 
 * 
 */
public class CameraPreview extends SurfaceView implements Callback {
	private Camera mCamera;
	private SurfaceHolder mSurfaceHolder;

	private int mScreenWidth, mScreenHeight;

	/**
	 * 当打开预览和关闭预览的时候应该设置这个状态值。
	 */
	public boolean isPreview = false;

	/**
	 * @param context
	 *            activity实例
	 * @param camera
	 *            相机实例
	 * @param screenWidth
	 *            预览宽度
	 * @param screenHeight
	 *            预览高度
	 */
	public CameraPreview(Context context, Camera camera, int screenWidth,
			int screenHeight) {
		super(context);
		mScreenHeight = screenHeight;
		mScreenWidth = screenWidth;

		mCamera = camera;
		mSurfaceHolder = getHolder();

		// 这里注册了一个回调
		mSurfaceHolder.addCallback(this);
		// 设置了显示图像的方式
		mSurfaceHolder.setType(SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS);
	}

	// 下面是对应的callback回调的重载函数
	
	public void surfaceChanged(SurfaceHolder holder, int format, int width,
			int height) {
		// TODO Auto-generated method stub

	}

	
	public void surfaceCreated(SurfaceHolder holder) {

		initCamera();

	}

	
	public void surfaceDestroyed(SurfaceHolder holder) {
		if (mCamera != null) {
			if (isPreview) {
				mCamera.stopPreview();
				mCamera.release();
				mCamera = null;
			}
		}

	}

	/**
	 * 设置相机参数
	 */
	private void initCamera() {
		if (mCamera == null) {

		}

		if (mCamera != null && !isPreview) {
			try {
				Camera.Parameters parameters = mCamera.getParameters();
				parameters.setPreviewSize((int) (mScreenWidth),
						(int) (mScreenHeight));
				parameters.setPreviewFrameRate(4);
				//Size pictureSize = findBestPictureSize(parameters);
				
				parameters.setPictureFormat(PixelFormat.JPEG);
				parameters.set("jpeg-quality", 100);
                parameters.setFlashMode(Camera.Parameters.FLASH_MODE_AUTO);
                parameters.setFocusMode(Camera.Parameters.FOCUS_MODE_AUTO);
                parameters.setJpegQuality(100);
				parameters.setPictureSize(mScreenWidth, mScreenHeight);
                //parameters.setPictureSize(1280, 720);
				mCamera.setPreviewDisplay(mSurfaceHolder);
				mCamera.startPreview();

				// mCamera.autoFocus(null);

			} catch (Exception e) {
				e.printStackTrace();
			}

			isPreview = true;
		}

	}

	private Size findBestPictureSize(Parameters parameters) {
		// TODO Auto-generated method stub
		String previewSizeValueString = null;
		int diff = Integer.MAX_VALUE;
		previewSizeValueString = parameters.get("preview-size-values");
		if (previewSizeValueString == null)
		{             
			previewSizeValueString = parameters.get("preview-size-value");
		}
		if(previewSizeValueString == null)
		{  // 有些手机例如m9获取不到支持的预览大小   就直接返回屏幕大小
			return  mCamera.new Size(getScreenWH().widthPixels,getScreenWH().heightPixels);
		}         
		Log.d("tag", "previewSizeValueString : " + previewSizeValueString);
		int bestX = 0;
		int bestY = 0;
		for(String prewsizeString : previewSizeValueString.split(":"))
		{             
			prewsizeString = prewsizeString.trim();
			int dimPosition = prewsizeString.indexOf('x');
			if(dimPosition == -1)
			{                 
				Log.e("t", "Bad prewsizeString:"+prewsizeString); 
				continue;
			}
			int newX = 0;             
			int newY = 0;                         
			try{                 
				newX = Integer.parseInt(prewsizeString.substring(0, dimPosition));   
				newY = Integer.parseInt(prewsizeString.substring(dimPosition+1));         
				}catch(NumberFormatException e){
					Log.e("t", "Bad prewsizeString:"+prewsizeString); 
					continue;             
					}                        
			Point screenResolution = new Point (getScreenWH().widthPixels,getScreenWH().heightPixels);
			int newDiff = Math.abs(newX - screenResolution.x)+Math.abs(newY- screenResolution.y);  
			if(newDiff == diff)             
			{               
				bestX = newX;          
				bestY = newY;           
				break;          
			} else if(newDiff < diff)
			{     
				if((3 * newX) == (4 * newY))
				{     
					bestX = newX;   
					bestY = newY;                 
					diff = newDiff;                
				}           
			}           
		}                  
		if (bestX > 0 && bestY > 0)
		{         
			
			return mCamera.new Size(bestX, bestY);   
		}
		return null;
	}

					
	private DisplayMetrics  getScreenWH() {
		// TODO Auto-generated method stub
		  DisplayMetrics dMetrics = new DisplayMetrics();
		  dMetrics = this.getResources().getDisplayMetrics();
		  return dMetrics;
	}
}
