package qsl.learn;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.DialogInterface.OnClickListener;
import android.database.Cursor;
import android.graphics.Bitmap;
import android.graphics.Bitmap.CompressFormat;
import android.graphics.Bitmap.Config;
import android.graphics.Color;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageView;

/**
 * 
 * 特效页面，实现了对一张图片的高斯模糊，复古化，柱面化得效果。
 * 
 * @author i0345
 * 
 */
public class EffectPhotoActivity extends Activity {
	private ImageView mImageView;
	private Bitmap mBitmap;

	@Override
	protected void onCreate(Bundle savedInstanceState) {

		super.onCreate(savedInstanceState);
		setContentView(R.layout.effect);

		mImageView = (ImageView) findViewById(R.id.effectview);
		long id = getIntent().getLongExtra(PhotoDbAdapter.KEY_ROWID, -1);

		if (id != -1) {
			PhotoDbAdapter photoDbAdapter = new PhotoDbAdapter(this);
			photoDbAdapter.open();
			Cursor cursor = photoDbAdapter.fetchPhoto(id);
			if (cursor.moveToFirst()) {
				String path = cursor.getString(cursor
						.getColumnIndex(PhotoDbAdapter.KEY_PATH));
				Log.d("EffectPhotoActivity", "get the path:" + path);

				mBitmap = PhotoUtil.decodeBitmap(path, 400);
				// mBitmap = oldRemember();
				mImageView.setImageBitmap(mBitmap);

			}
			cursor.close();
			cursor = null;
			photoDbAdapter.close();

		} else {
			// TODO deal with the erro ,it don't get the id
			Log.d("erro", "No Rowid");
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		MenuInflater menuInflater = new MenuInflater(this);
		menuInflater.inflate(R.menu.effect_menu, menu);
		return super.onCreateOptionsMenu(menu);

	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case R.id.effect_item1:
			new OldEffectTask().execute(mBitmap);

			break;
		case R.id.effect_item2:
			new blurTask().execute(mBitmap);
			break;
		case R.id.effect_item3:
			new CylinderFromRectangleTask().execute(mBitmap);
			break;

		case R.id.effect_item4:
			createDialog();
			break;
		default:
			break;
		}

		return super.onOptionsItemSelected(item);
	}

	private void createDialog() {
		// TODO Auto-generated method stub\
		View saveDialog = getLayoutInflater().inflate(R.layout.save, null);
		final EditText photonameEditText = (EditText) saveDialog
				.findViewById(R.id.phone_name);
		ImageView imageView = (ImageView) saveDialog.findViewById(R.id.show);
		imageView.setImageBitmap(mBitmap);
		new AlertDialog.Builder(EffectPhotoActivity.this)
				.setView(saveDialog)
				.setPositiveButton(getString(R.string.text_save),
						new OnClickListener() {

							
							public void onClick(DialogInterface dialog,
									int which) {
								String name = photonameEditText.getText()
										.toString();
								savePicture(name, mBitmap);

							}
						})
				.setNegativeButton(getString(R.string.text_cancel),
						new OnClickListener() {

							
							public void onClick(DialogInterface dialog,
									int which) {
								// TODO Auto-generated method stub

							}
						}).show();
	}

	private long savePicture(String name, Bitmap bm) {

		String pathString = (name == "" ? Const.PHOTO_NAME_STRING : name
				+ ".jpg");
		File dirFile = new File(Const.PIC_PATH);
		long RowId = 0;
		if (!dirFile.exists()) {

			dirFile.mkdirs();

		}
		File file = new File(Const.PIC_PATH, pathString);
		BufferedOutputStream bos = null;
		try {
			bos = new BufferedOutputStream(new FileOutputStream(file));

			bm.compress(CompressFormat.JPEG, 100, bos);
			bos.flush();
			bos.close();
			PhotoDbAdapter photoDbAdapter = new PhotoDbAdapter(
					EffectPhotoActivity.this);
			photoDbAdapter.open();
			RowId = photoDbAdapter.createPhoto(file.getName(), file.getPath());
			photoDbAdapter.close();
		} catch (Exception e) {

		} finally {
			bos = null;
		}
		return RowId;
	}

	private class blurTask extends AsyncTask<Bitmap, Void, Bitmap> {
		ProgressDialog mProgressDialog;

		@Override
		protected Bitmap doInBackground(Bitmap... params) {
			Bitmap result = blurImageAmeliorate(params[0]);
			return result;
		}

		@Override
		protected void onPreExecute() {
			mProgressDialog = ProgressDialog.show(EffectPhotoActivity.this,
					"正在处理图像", "请耐心等待", true, false);
		}

		@Override
		protected void onPostExecute(Bitmap result) {
			mProgressDialog.dismiss();
			mBitmap.recycle();
			mBitmap = result;
			mImageView.setImageBitmap(mBitmap);
			mImageView.invalidate();

		}

	}

	private Bitmap blurImageAmeliorate(Bitmap taskBitmap) {
		long start = System.currentTimeMillis();
		int[] gauss = new int[] { 1, 2, 1, 2, 4, 2, 1, 2, 1 };
		int delta = 16;
		int width = mBitmap.getWidth();
		int height = mBitmap.getHeight();
		Bitmap bitmap = Bitmap.createBitmap(width, height, Config.RGB_565);

		int pixR = 0;
		int pixG = 0;
		int pixB = 0;

		int pixColor = 0;

		int newR = 0;
		int newG = 0;
		int newB = 0;

		int idx = 0;

		int[] pixels = new int[width * height];
		taskBitmap.getPixels(pixels, 0, width, 0, 0, width, height);

		for (int i = 1, heightLen = height - 1; i < heightLen; i++) {
			for (int j = 1, widthLen = width - 1; j < widthLen; j++) {
				idx = 0;
				for (int m = -1; m <= 1; m++) {
					for (int n = -1; n <= 1; n++) {
						pixColor = pixels[(i + m) * width + j + n];
						pixR = Color.red(pixColor);
						pixG = Color.green(pixColor);
						pixB = Color.blue(pixColor);

						newR = newR + (int) pixR * gauss[idx];
						newG = newG + (int) pixG * gauss[idx];
						newB = newB + (int) pixB * gauss[idx];
						idx++;
					}
				}
				newR /= delta;
				newG /= delta;
				newB /= delta;

				newR = Math.min(255, Math.max(0, newR));
				newG = Math.min(255, Math.max(0, newG));
				newB = Math.min(255, Math.max(0, newB));

				pixels[i * width + j] = Color.argb(255, newR, newG, newB);

				newR = 0;
				newG = 0;
				newB = 0;
			}
		}
		bitmap.setPixels(pixels, 0, width, 0, 0, width, height);
		long end = System.currentTimeMillis();
		Log.d("cost_time", "cost_time:" + (end - start));

		return bitmap;

	}

	private class OldEffectTask extends AsyncTask<Bitmap, Void, Bitmap> {
		ProgressDialog mProgressDialog;

		@Override
		protected Bitmap doInBackground(Bitmap... params) {
			Bitmap result = oldRemember(params[0]);
			return result;
		}

		@Override
		protected void onPreExecute() {
			mProgressDialog = ProgressDialog.show(EffectPhotoActivity.this,
					"正在处理图像", "请耐心等待", true, false);
		}

		@Override
		protected void onPostExecute(Bitmap result) {
			mProgressDialog.dismiss();
			mBitmap.recycle();
			mBitmap = result;
			mImageView.setImageBitmap(mBitmap);
			mImageView.invalidate();
		}

	}

	private Bitmap oldRemember(Bitmap taskBitmap) {
		long start = System.currentTimeMillis();
		int width = mBitmap.getWidth();
		int height = mBitmap.getHeight();
		Bitmap bitmap = Bitmap.createBitmap(width, height,
				Bitmap.Config.RGB_565);
		int pixcolor = 0;
		int pixR = 0;
		int pixG = 0;
		int pixB = 0;
		int newR = 0;
		int newG = 0;
		int newB = 0;
		int[] pixels = new int[width * height];
		taskBitmap.getPixels(pixels, 0, width, 0, 0, width, height);
		for (int i = 0; i < height; i++) {
			for (int j = 0; j < width; j++) {
				pixcolor = pixels[width * i + j];
				pixR = Color.red(pixcolor);
				pixG = Color.green(pixcolor);
				pixB = Color.blue(pixcolor);
				newR = (int) (0.393 * pixR + 0.769 * pixG + 0.189 * pixB);
				newG = (int) (0.349 * pixR + 0.686 * pixG + 0.186 * pixB);
				newB = (int) (0.272 * pixR + 0.534 * pixG + 0.131 * pixB);
				int newColor = Color.argb(255, newR > 255 ? 255 : newR,
						newG > 255 ? 255 : newG, newB > 255 ? 255 : newB);
				pixels[width * i + j] = newColor;
			}

		}
		bitmap.setPixels(pixels, 0, width, 0, 0, width, height);
		long end = System.currentTimeMillis();
		Log.d("effect_cost_time", "costTime:" + (end - start));
		return bitmap;
	}

	private class CylinderFromRectangleTask extends
			AsyncTask<Bitmap, Void, Bitmap> {
		ProgressDialog mProgressDialog;

		@Override
		protected Bitmap doInBackground(Bitmap... params) {
			Bitmap result = CylinderFromRectangle(params[0]);
			return result;
		}

		@Override
		protected void onPreExecute() {
			mProgressDialog = ProgressDialog.show(EffectPhotoActivity.this,
					"正在处理图像", "请耐心等待", true, false);
		}

		@Override
		protected void onPostExecute(Bitmap result) {
			mProgressDialog.dismiss();
			mBitmap.recycle();
			mBitmap = result;
			mImageView.setImageBitmap(mBitmap);
			mImageView.invalidate();
		}

	}

	private Bitmap CylinderFromRectangle(Bitmap taskBitmap) {
		long start = System.currentTimeMillis();
		int width = mBitmap.getWidth();
		int height = mBitmap.getHeight();
		Bitmap bitmap = Bitmap.createBitmap(width, height,
				Bitmap.Config.RGB_565);
		double r = (double) width / Math.tan(Math.toRadians(30));
		Log.d("半径：", "半径：" + r);
		double a;
		int x;
		int y;
		int pixcolor = 0;
		int idx;
		int idx2;
		int[] pixels = new int[width * height];
		int[] pixelsResult = new int[width * height];
		taskBitmap.getPixels(pixels, 0, width, 0, 0, width, height);

		for (int i = -width / 2; i < width / 2; i++) {
			a = Math.atan(i / r);
			x = (int) (a * r);
			for (int j = height / 2; j > -height / 2; j--) {
				y = (int) (j * Math.cos(a));
				idx = (i + width / 2) + (height / 2 - j) * width;
				idx = Math.min(width * height - 1, Math.max(0, idx));
				pixcolor = pixels[idx];
				idx2 = x + width / 2 + (height / 2 - y) * width;
				idx2 = Math.min(width * height - 1, Math.max(0, idx2));
				pixelsResult[idx2] = pixcolor;
			}
		}
		pixels = null;
		bitmap.setPixels(pixelsResult, 0, width, 0, 0, width, height);
		long end = System.currentTimeMillis();
		Log.d("effect_cost_time", "costTime:" + (end - start));
		return bitmap;
	}

	@Override
	protected void onDestroy() {
		if (mBitmap == null) {
			mBitmap.recycle();
		}

		super.onDestroy();

	}
}
