package qsl.learn;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

/**
 * 封装了一个数据库，实现了对图片的名字和路径的存储。
 * 
 * 
 * 
 */
public class PhotoDbAdapter {
	public static final String KEY_NAME = "name";
	public static final String KEY_PATH = "path";
	public static final String KEY_ROWID = "_id";

	private static final String TAG = "PhotoDbAdapter";
	private DatabaseHelper mDbHelper;
	private SQLiteDatabase mDb;
	/**
	 * Database creation sql statement
	 */
	private static final String DATABASE_CREATE = "create table photos (_id integer primary key autoincrement, "
			+ "name text not null, path text not null);";

	private static final String DATABASE_NAME = "data";
	private static final String DATABASE_TABLE = "photos";
	private static final int DATABASE_VERSION = 2;

	private Context mCtx;

	class DatabaseHelper extends SQLiteOpenHelper {

		public DatabaseHelper(Context context) {
			super(context, DATABASE_NAME, null, DATABASE_VERSION);

		}

		@Override
		public void onCreate(SQLiteDatabase db) {
			db.execSQL(DATABASE_CREATE);

		}

		@Override
		public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
			Log.w(TAG, "Upgrading database from version " + oldVersion + " to "
					+ newVersion + ", which will destroy all old data");
			db.execSQL("DROP TABLE IF EXISTS photos");
			onCreate(db);

		}
	}

	public PhotoDbAdapter(Context context) {
		this.mCtx = context;

	}

	public PhotoDbAdapter open() throws SQLException {
		mDbHelper = new DatabaseHelper(mCtx);
		mDb = mDbHelper.getWritableDatabase();
		return this;
	}

	public void close() {
		mDbHelper.close();
	}

	public long createPhoto(String name, String path) {
		ContentValues initialValues = new ContentValues();
		initialValues.put(KEY_NAME, name);
		initialValues.put(KEY_PATH, path);

		return mDb.insert(DATABASE_TABLE, null, initialValues);
	}

	public boolean deletePhoto(long rowId) {

		return mDb.delete(DATABASE_TABLE, KEY_ROWID + "=" + rowId, null) > 0;
	}

	public Cursor fetchAllPhotos() {

		return mDb.query(DATABASE_TABLE, new String[] { KEY_ROWID, KEY_NAME,
				KEY_PATH }, null, null, null, null, null);
	}

	public Cursor fetchPhoto(long rowId) throws SQLException {

		Cursor mCursor =

		mDb.query(true, DATABASE_TABLE, new String[] { KEY_ROWID, KEY_NAME,
				KEY_PATH }, KEY_ROWID + "=" + rowId, null, null, null, null,
				null);
		if (mCursor != null) {
			mCursor.moveToFirst();
		}
		return mCursor;

	}

	public boolean updateNote(long rowId, String title, String body) {
		ContentValues args = new ContentValues();
		args.put(KEY_NAME, title);
		args.put(KEY_PATH, body);

		return mDb.update(DATABASE_TABLE, args, KEY_ROWID + "=" + rowId, null) > 0;
	}

}
