#include "ImageFieldDepthMaker.h"

const int direct[4][2] = {{1, 0}, {-1, 0}, {0, 1}, {0, -1}};

ImageFieldDepthMaker::ImageFieldDepthMaker(){
	h_size = v_size = 0;
	MatchPointSet.clear();
}

void ImageFieldDepthMaker::CMDOperations(){
	int t;
	while(1){
		t = cvWaitKey(0);
		if((char)t == 'r')
			chopper.refresh();
		else if((char)t == 'w')
			chopper.chop();
		else if((char)t == 'o')
			break;
	}
}

ImageFieldDepthMaker::~ImageFieldDepthMaker(){
	flag_left.clear();
	MatchPointSet.clear();
}

// void ImageFieldDepthMaker::init(int spatialRad, int colorRad, int maxPryLevel){
// 	chopper.InitParameter(spatialRad, colorRad, maxPryLevel);
// }


bool ImageFieldDepthMaker::LoadPicLeftRight(const string& f_left, const string& f_right){
	exif.parseFrom(f_left);
	IplImage* t_l = cvLoadImage(f_left.c_str());
	if(t_l == NULL)
		return false;
	double sc = 1080.0 / (double)t_l->width;
	left = cvCreateImage(cvSize(t_l->width * sc, t_l->height * sc), t_l->depth, t_l->nChannels);
	cvResize(t_l, left);
	cvReleaseImage(&t_l);

	IplImage* t_r = cvLoadImage(f_right.c_str());
	if(t_r == NULL){
		cvReleaseImage(&left);
		return false;
	} else {
		right = cvCreateImage(cvSize(t_r->width * sc, t_r->height * sc), t_r->depth, t_r->nChannels);
		cvResize(t_r, right);
		cvReleaseImage(&t_r);
	}
	chopper.loadPicture(f_left.c_str());
	return true;
}

bool ImageFieldDepthMaker::getFieldDepth(PicDOFInfo** outDOFInfo){
	if(dof_info.x_lu == -1){
		outDOFInfo = NULL;
		return false;
	} else {
		*outDOFInfo = &dof_info;
		return true;
	}
}

static int mean(const vector<int>& v){
	int rslt = 0;
	size_t l = v.size();
	for(int i = 0; i < l; i++){
		rslt += v[i];
	}
	rslt /= l;
	return rslt;
}

static int min(const vector<int>& v){
	int rslt = 0x3ffffff;
	size_t l = v.size();
	for(int i = 0; i < l; i++){
		if(rslt > v[i])
			rslt = v[i];
	}
	return rslt;
}

static int max(const vector<int>& v){
	int rslt = -0x3ffffff;
	size_t l = v.size();
	for(int i = 0; i < l; i++){
		if(rslt < v[i])
			rslt = v[i];
	}
	return rslt;
}

bool ImageFieldDepthMaker::calFieldDepth(){
	map<bgr_point, int> me = chopper.getColorMap();
	IplImage c_l = IplImage(*(chopper.getMasker()));
	map<int, vector<int>> m;
	map<int, vector<int>>::iterator it;
	int l_b, dis, x_l = left->width, y_l = left->height, x_r = 0, y_r = 0;

	//chop and divide into blocks

	//TODO: delete stack DFS, modify to use watershed chopper's color map, 
	DFSSearch(&c_l, flag_left, lu_corner_l, rd_corner_l, me);

	//SIFT matching
	matcher.getMatchedPointDuals(MatchPointSet, left, right);
 	matcher.ransac(MatchPointSet, flag_left, me.size());
	int c_match = MatchPointSet.size();


	IplImage* dst = chopper.getDst();
	for(int i = 0; i < c_match; i++){
		l_b = flag_left[MatchPointSet[i].first.y][MatchPointSet[i].first.x];
		dis = MatchPointSet[i].first.x - MatchPointSet[i].second.x;
		if(dis < 0)
			continue;
		it = m.find(l_b);
		if(it == m.end()){
			m.insert(pair<int, vector<int>>(l_b, vector<int>()));
		}
		cvCircle(dst,
			cvPoint(MatchPointSet[i].first.x, MatchPointSet[i].first.y),
			3,
			cvScalar(255, 255, 255),
			1, 8, 3);
		m.find(l_b)->second.push_back(dis);
	}
	cvShowImage("watershed transform", dst );
	cvWaitKey();
	//cvSaveImage("chop_mark.jpg", dst);

	//find lu and rd corners
	printf("matched blocks: %d\n", m.size());
	it = m.begin();

	while(it!=m.end()){
		printf("%d matched with %d\n", it->first, max(it->second));
		pair<int, int> p = lu_corner_l[it->first];
		if(p.first < x_l)
			x_l = p.first;
		if(p.second < y_l)
			y_l = p.second;
		printf("%d %d ", p.first, p.second);

		p = rd_corner_l[it->first];
		if(p.first > x_r)
			x_r = p.first;
		if(p.second > y_r)
			y_r = p.second;
		printf("%d %d\n", p.first, p.second);
		++it;
	}
	dof_info.x_lu = x_l;
	dof_info.x_rd = x_r;
	dof_info.y_lu = y_l;
	dof_info.y_rd = y_r;
	dof_info.blockMap = flag_left;
	dof_info.DOFMat = cvCreateMat(y_r - y_l, x_r - x_l, CV_32FC1);
	float* data = dof_info.DOFMat->data.fl;
	int step = dof_info.DOFMat->step / sizeof(float);
	for(int i = y_l; i < y_r; i++){
		for(int j = x_l; j < x_r; j++){
			it = m.find(flag_left[i][j]);
			data[(i - y_l) * step + j - x_l] = (it == m.end() ? 0.0 : (exif.FocalLength * 8000.0 / max(it->second)));
		}
	}

	vector<pair<CvPoint,CvPoint>> dir_arr = chopper.getDirArr();
	vector<int> isDired;
	isDired.resize(me.size());
	vector<pair<float, float>> max_min_dir;
	int block;
	max_min_dir.resize(me.size());

	for(int i = 0; i < isDired.size(); i++){
		isDired[i] = 0;
	}

	for(int i = 0; i < dir_arr.size(); i++){
		block = flag_left[dir_arr[i].first.y][dir_arr[i].first.x] - 1;
		isDired[block] = dir_arr[i].first.y - dir_arr[i].second.y;
		max_min_dir[block].first = -1;
		max_min_dir[block].second = 0x3fffff;
	}

	//finding distance interval
	for(int i = 0; i < isDired.size(); i++){
		if(isDired[i] != 0){
			pair<int, int>& p_l = lu_corner_l[i];
			pair<int, int>& p_r = rd_corner_l[i];
			int x_start = p_l.first, y_start = p_r.second, x_end = p_r.first, y_end = p_l.second;
			for(int j = y_start; j >= y_end; j--){
				for(int k = x_start; k <= x_end; k++){
					if(max_min_dir[i].first < data[(j-y_l) * step + k - x_l]){
						max_min_dir[i].first = data[(j-y_l) * step + k - x_l];
					} else if (max_min_dir[i].second > data[(j-y_l) * step + k - x_l]){
						max_min_dir[i].second = data[(j-y_l) * step + k - x_l];
					}
				}
			}
		}
	}

	for(int i = 0; i < isDired.size(); i++){
		pair<int, int>& p_l = lu_corner_l[i];
		pair<int, int>& p_r = rd_corner_l[i];
		int x_start, y_start, x_end, y_end;
		if(isDired[i] < 0){
			x_start = p_l.first;
			y_start = p_r.second;
			x_end = p_r.first;
			y_end = p_l.second;
			float e = max_min_dir[i].first, s = max_min_dir[i].second, gaps = (e - s) * 0.05;
			e = e - gaps;
			s = s + gaps;
			gaps = (e - s) / (abs(y_end - y_start));
			printf("down: start: %d %d %f, end: %d %d %f, gaps: %f.\n", y_start, x_start, s, y_end, x_end, e, gaps);
			for(int j = y_start; j >= y_end; j--){
				for(int k = x_start; k <= x_end; k++){
					if(flag_left[j][k] == i + 1)
						data[(j - y_l) * step + k - x_l] = e;
				}
				e = e - gaps;
			}
		} else if(isDired[i] > 0){
			x_start = p_l.first;
			y_start = p_l.second;
			x_end = p_r.first;
			y_end = p_r.second;
			float e = max_min_dir[i].first, s = max_min_dir[i].second, gaps = (e - s) * 0.05;
			e = e - gaps;
			s = s + gaps;
			gaps = (e - s) / (abs(y_end - y_start));
			printf("start: %d %d %f, end: %d %d %f, gaps: %f.\n", y_start, x_start, s, y_end, x_end, e, gaps);
			for(int j = y_start; j <= y_end; j++){
				for(int k = x_start; k <= x_end; k++){
					if(flag_left[j][k] == i + 1){
						data[(j - y_l) * step + k - x_l] = e;
					}
				}
				e = e - gaps;
			}
		}
	}

	return true;
}

void ImageFieldDepthMaker::DFSSearch(
	IplImage* img,
	vector<vector<unsigned int>>& mat,
	vector<pair<int, int>>& v_lu,
	vector<pair<int, int>>& v_rd,
	map<bgr_point, int> colorMap){
	int row = img->height, col = img->width, block;
	map<bgr_point, int>::iterator it;

	for(int i = 0; i < row; i++){
		mat.push_back(vector<unsigned int>());
		mat[i].resize(col);
		for(int j = 0; j < col; j++)
			mat[i][j] = 0;
	}
	h_size = col;
	v_size = row;

	block = colorMap.size();
	for(int i = 0; i < block; i++){
		v_lu.push_back(pair<int, int>(col, row));
		v_rd.push_back(pair<int, int>(-1, -1));
	}

	for(int i = 0; i < row; i++){
		for(int j = 0; j < col; j++){
			it = colorMap.find(getBGR(img, j, i));
			if(it != colorMap.end()){
				block = it->second - 1;
				mat[i][j] = block + 1;
				if(v_lu[block].first > j)
					v_lu[block].first = j;
				if(v_lu[block].second > i)
					v_lu[block].second = i;
				if(v_rd[block].first < j)
					v_rd[block].first = j;
				if(v_rd[block].second < i)
					v_rd[block].second = i;
			} else {
				mat[i][j] = 0;
			}
		}
	}
}

bgr_point ImageFieldDepthMaker::getBGR(IplImage* img, int x, int y){
	unsigned char* data = (unsigned char*)img->imageData + y * img->widthStep + x * 3;
	return bgr_point(data[0], data[1], data[2]);
}