#ifndef IMAGE_FIELD_DEPTH_MAKER_H
#define IMAGE_FIELD_DEPTH_MAKER_H

#include "config.h"
#include "MeanShiftChopper.h"
#include "WatershedChopper.h"
#include "SIFTMatcher.h"
#include "exif.h"

//#define DEBUG

#ifdef _DEBUG
#pragma comment(lib, "opencv_core244d.lib")
#pragma comment(lib, "opencv_calib3d244d.lib")
#pragma comment(lib, "opencv_highgui244d.lib")
#pragma comment(lib, "opencv_imgproc244d.lib")
#else
#pragma comment(lib, "opencv_core244.lib")
#pragma comment(lib, "opencv_calib3d244.lib")
#pragma comment(lib, "opencv_highgui244.lib")
#pragma comment(lib, "opencv_imgproc244.lib")
#endif

typedef struct _PicDOFInfo{
	int x_lu;
	int y_lu;
	int x_rd;
	int y_rd;
	double camera_f;
	CvMat* DOFMat;
	vector<vector<unsigned int>> blockMap;
	_PicDOFInfo(){
		x_lu = y_lu = x_rd = y_rd = -1;
		DOFMat = NULL;
	}
}PicDOFInfo;

class ImageFieldDepthMaker{
public:
	//load picture from different point of view
	bool LoadPicLeftRight(const string& f_left, const string& f_right);

	//interface to get the depth of field
	bool getFieldDepth(PicDOFInfo** outDOFInfo);

	//calculate the field depth, called before getFieldDepth;
	bool calFieldDepth();

// 	void init(int spatialRad, int colorRad, int maxPryLevel);

	//construction method
	ImageFieldDepthMaker();

	void CMDOperations();

	//release resources
	~ImageFieldDepthMaker();
private:
	void DFSSearch(
		IplImage* img,
		vector<vector<unsigned int>>& pMat,
		vector<pair<int, int>>& v_lu,
		vector<pair<int, int>>& v_rd,
		map<bgr_point, int> colorMap
		);

	vector<vector<unsigned int>> flag_left;

	unsigned int h_size, v_size;

	vector<pair<point, point>> MatchPointSet;

	IplImage *left, *right;

	Mat ch_l;

	EXIFInfo exif;

	vector<pair<int, int>> lu_corner_l, rd_corner_l;

	bgr_point getBGR(IplImage* img, int x, int y);

	PicDOFInfo dof_info;

	SIFTMatcher matcher;

	WatershedChopper chopper;
};

#endif