#include "ImgDoFEffectMaker.h"

#include <cstdio>

bool ImgDoFEffectMaker::loadDepInfo(const PicDOFInfo &dep) {
	int i, j;
	double depMax = 0;
	CocMap mp;
	CocMap::iterator it;
	
	if (!dep.DOFMat) return false;
	
	m_f = 0.4;

	//FILE *fp = fopen("load.out", "w");

	//fprintf(fp, "%d %d %d %d\n", dep.x_lu, dep.y_lu, dep.x_rd, dep.y_rd);

	m_lu.x = dep.x_lu; m_lu.y = dep.y_lu;
	m_rd.x = dep.x_rd; m_rd.y = dep.y_rd;
	m_row = dep.y_rd - dep.y_lu;
	m_col = dep.x_rd - dep.x_lu;

	m_dstImg.create(m_row, m_col, CV_8UC3);

	m_dep.resize(m_row, m_col);
	m_u = DISTANCE_INF;
	for (i = 0; i < m_row; ++i) {
		for (j = 0; j < m_col; ++j) {
			m_dep[i][j] = dep.DOFMat->data.fl[i * m_col + j];

			if (zero(m_dep[i][j])) {
				//if (i > 0 && j > 0) m_dep[i][j] = (m_dep[i - 1][j] + m_dep[i][j - 1]) / 2.0;
				//else if (i > 0) m_dep[i][j] = m_dep[i - 1][j];
				//else if (j > 0) m_dep[i][j] = m_dep[i][j - 1];
			} else {
				depMax = std::max(depMax, m_dep[i][j]);
				//if (m_dep[i][j] > eps) m_u = std::min(m_u, m_dep[i][j]);
				it = mp.find(m_dep[i][j]);
				if (it != mp.end()) ++it->second;
				else mp.insert(std::make_pair(m_dep[i][j], 1));
			}

			//fprintf(fp, "%lf ", m_dep[i][j]);

			depMax = std::max(depMax, m_dep[i][j]);
			if (m_dep[i][j] > eps) m_u = std::min(m_u, m_dep[i][j]);
		}
		
		//fprintf(fp, "\n");
	}

	//fclose(fp);
	// doErosion();

	for (it = mp.begin(); it != mp.end(); ++it) {
		if (it->second > (m_row * m_col * AREA_RATE)) {
			m_u = it->first;
			break;
		}
	}

	m_u = m_dep[270][591];

	for (i = 0; i < m_row; ++i) {
		for (j = 0; j < m_col; ++j) {
			m_dep[i][j] /= depMax;
		}
	}

	m_u /= depMax;
	m_f /= (depMax * 1000.0);

	return true;
}

bool ImgDoFEffectMaker::loadDepInfo(const std::string &file) {
	int i, j, cnt;
	double depMax = 0;
	FILE *fp = fopen(file.c_str(), "r");
	CocMap mp;
	CocMap::iterator it;

	if (!fp) return false;
	
	m_f = 0.4;

	fscanf(fp, "%d %d %d %d", &m_lu.x, &m_lu.y, &m_rd.x, &m_rd.y);

	m_row = m_rd.y - m_lu.y;
	m_col = m_rd.x - m_lu.x;

	m_dstImg.create(m_row, m_col, CV_8UC3);

	m_dep.resize(m_row, m_col);
	m_u = DISTANCE_INF;

	m_vis.resize(m_row, m_col);
	m_vis.fill(false);

	for (i = 0; i < m_row; ++i) {
		for (j = 0; j < m_col; ++j) {
			fscanf(fp, "%lf", &m_dep[i][j]);
			if (!zero(m_dep[i][j])) m_vis[i][j] = true;
		}
	}

	//doErosion();

	mp.clear();
	for (i = 0; i < m_row; ++i) {
		for (j = 0; j < m_col; ++j) {
			if (zero(m_dep[i][j])) {
				cnt = 0;
				if (i > 0 && m_vis[i - 1][j]) {
					m_dep[i][j] += m_dep[i - 1][j];
					++cnt;
				}
				if (i < m_row - 1 && m_vis[i + 1][j]) {
					m_dep[i][j] += m_dep[i + 1][j];
					++cnt;
				}
				if (j > 0 && m_vis[i][j - 1]) {
					m_dep[i][j] += m_dep[i][j - 1];
					++cnt;
				}
				if (j < m_col - 1 && m_vis[i][j + 1]) {
					m_dep[i][j] += m_dep[i][j + 1];
					++cnt;
				}
				m_dep[i][j] /= cnt;
			}


			depMax = std::max(depMax, m_dep[i][j]);
			it = mp.find(m_dep[i][j]);
			if (it != mp.end()) ++it->second;
			else mp.insert(std::make_pair(m_dep[i][j], 1));
		}
	}

	for (it = mp.begin(); it != mp.end(); ++it) {
		if (it->second > (m_row * m_col * AREA_RATE)) {
			m_u = it->first;
			break;
		}
	}

	//m_u = m_dep[270][591];

	for (i = 0; i < m_row; ++i) {
		for (j = 0; j < m_col; ++j) {
			m_dep[i][j] /= depMax;
		}
	}

	printf("m_u = %.8lf, depMax = %lf\n", m_u, depMax);

	m_u /= depMax;
	m_f /= (depMax * 1000.0);

	fclose(fp);

	return true;
}

void ImgDoFEffectMaker::makeDoFEffect(double diam) {
	int i, j;
	clock_t start, stop;

	initCoC(diam);

	m_dis.resize(m_row, m_col);
	m_dis.fill(-1);
	for (i = 0; i < m_row; ++i) {
		start = clock();
		for (j = 0; j < m_col; ++j) {
			addEffect(PixPoint(j, i));
		}
		stop = clock();
		printf("i = %d, time = %d\n", i, stop - start);
	}
}

void ImgDoFEffectMaker::initCameraParams(const std::string &src) {
	//m_exifInfo.parseFrom(src);
	//m_f = 4;
}

void ImgDoFEffectMaker::initCoC(double diam) {
	int i, j;
	double coc_max = 0;

	FILE *fp = fopen("coc.out", "w");

	m_coc.resize(m_row, m_col);
	for (i = 0; i < m_row; ++i) {
		for (j = 0; j < m_col; ++j) {
			m_coc[i][j] = calcCoC(PixPoint(j, i), diam);
			if (m_coc[i][j] < DISTANCE_INF) coc_max = std::max(coc_max, m_coc[i][j]);
			
		}
	}

	for (i = 0; i < m_row; ++i) {
		for (j = 0; j < m_col; ++j) {
			if (m_coc[i][j] < DISTANCE_INF) m_coc[i][j] = m_coc[i][j] / coc_max * KERNEL_RADIUS;
			else m_coc[i][j] = KERNEL_RADIUS;
			fprintf(fp, "%lf ", m_coc[i][j]);
		}
		fprintf(fp, "\n");
	}

	doGauss(m_coc);

	fclose(fp);
}

void ImgDoFEffectMaker::addEffect(const PixPoint &p) {
	PixPoint cur, nxt;
	double color[3] = {0.0, 0.0, 0.0}, sum = 0.0; 
	int i, cnt = 0;

	m_bfsque.clear();
	m_dis[p.y][p.x] = 0; 
	m_bfsque.push(p); m_vispix[cnt++] = p;

	while (!m_bfsque.empty()) {
		cur = m_bfsque.pop();

		if (dis(cur, p) <= m_coc[cur.y][cur.x] + eps) {
			color[0] += static_cast<double>(cvMatPointAt<uchar>(m_srcImg, m_lu.y + cur.y, m_lu.x + cur.x, 0)) / 255.0;
			color[1] += static_cast<double>(cvMatPointAt<uchar>(m_srcImg, m_lu.y + cur.y, m_lu.x + cur.x, 1)) / 255.0;
			color[2] += static_cast<double>(cvMatPointAt<uchar>(m_srcImg, m_lu.y + cur.y, m_lu.x + cur.x, 2)) / 255.0;
			sum += 1.0;
		}
		
		if (m_dis[cur.y][cur.x] >= KERNEL_RADIUS) continue;
		for (i = 0; i < 8; ++i) {
			nxt = cur + step[i];
			if (!isInValidArea(nxt) || ~m_dis[nxt.y][nxt.x]) continue;
			m_dis[nxt.y][nxt.x] = m_dis[cur.y][cur.x] + 1;
			m_bfsque.push(nxt); m_vispix[cnt++] = nxt;
		}
	}

	color[0] /= sum; color[1] /= sum; color[2] /= sum;
	cvMatPointAt<uchar>(m_dstImg, cur.y, cur.x, 0) = static_cast<uchar>(color[0] * 255.0);
	cvMatPointAt<uchar>(m_dstImg, cur.y, cur.x, 1) = static_cast<uchar>(color[1] * 255.0);
	cvMatPointAt<uchar>(m_dstImg, cur.y, cur.x, 2) = static_cast<uchar>(color[2] * 255.0);

	for (i = 0; i < cnt; ++i) m_dis[m_vispix[i].y][m_vispix[i].x] = -1;
}

void ImgDoFEffectMaker::doErosion() {
	int i, j;
	CMatrix<double> dep_tmp(m_dep.row(), m_dep.col());

	dep_tmp.fill(0);
	for (i = 0; i < m_row; ++i) {
		for (j = 0; j < m_col; ++j) {
			if (zero(m_dep[i][j])) doErosionAt(m_dep, dep_tmp, PixPoint(j, i));
			else dep_tmp[i][j] = m_dep[i][j];
		}
	}
	for (i = 0; i < m_row; ++i) {
		for (j = 0; j < m_col; ++j) {
			m_dep[i][j] = dep_tmp[i][j];
		}
	}
}

void ImgDoFEffectMaker::doErosionAt(CMatrix<double> &src, CMatrix<double> &dst, const PixPoint &p) {
	int i, cnt = 0;
	PixPoint cur;

	for (i = 0; i < 8; ++i) {
		cur = p + step[i];
		if (isInValidArea(cur) && !zero(m_dep[cur.y][cur.x])) {
			dst[p.y][p.x] += src[cur.y][cur.x]; ++cnt;
		}
	}
	if (cnt) dst[p.y][p.x] /= (double)cnt;
}

void ImgDoFEffectMaker::doGauss(CMatrix<double> &src) {
	int i, j, k;
	CMatrix<double> tmp(src.row(), src.col());
	PixPoint cur;

	for (i = 0; i < tmp.row(); ++i) {
		for (j = 0; j < tmp.col(); ++j) {
			tmp[i][j] = src[i][j];
			for (k = 0; k < 8; ++k) {
				cur = PixPoint(j, i) + step[k];
				if (isInValidArea(cur)) tmp[i][j] += src[cur.y][cur.x];
			}
			tmp[i][j] /= 9;
		}
	}
	for (i = 0; i < tmp.row(); ++i) {
		for (j = 0; j < tmp.col(); ++j) {
			src[i][j] = tmp[i][j];
		}
	}
}