#include "SIFTMatcher.h"

/* the maximum number of keypoint NN candidates to check during BBF search */
#define KDTREE_BBF_MAX_NN_CHKS 200

/* threshold on squared ratio of distances between NN and 2nd NN */
#define NN_SQ_DIST_RATIO_THR 0.8

static double e_dis(CvPoint2D32f x, CvPoint2D32f y){
	return (x.x - y.x) * (x.x - y.x) +
		(x.y - y.y) * (x.y - y.y);
}
int myrandom(int i){
	return std::rand() % i;
}
void RANSAC_pair(vector<pair<CvPoint, CvPoint>> point_set, vector<int>& rsltId){
	size_t l = point_set.size();
	srand(unsigned(std::time(0)));
	size_t count = l * (l-1) * (l-2);
	unsigned char* r = new unsigned char[count];
	memset(r, 0, count);
	vector<int> id;
	int threshold = 2500;
	CvPoint2D32f src[3], dest[3], dst;
	for(int i = 0; i < l; i++)
		id.push_back(i);
	for(int i = 0; i < l/2; i++){
		random_shuffle(id.begin(), id.end());
		for(int j = 0; j < l - 2; j++){
			int x = id[j] * l * l + id[j+1] * l + id[j+2];
			if(x > count - 1 || r[x] != 0)
				continue;
			src[0].x = point_set[id[j]].first.x;
			src[0].y = point_set[id[j]].first.y;
			src[1].x = point_set[id[j+1]].first.x;
			src[1].y = point_set[id[j+1]].first.y;
			src[2].x = point_set[id[j+2]].first.x;
			src[2].y = point_set[id[j+2]].first.y;
			dest[0].x = point_set[id[j]].second.x;
			dest[0].y = point_set[id[j]].second.y;
			dest[1].x = point_set[id[j+1]].second.x;
			dest[1].y = point_set[id[j+1]].second.y;
			dest[2].x = point_set[id[j+2]].second.x;
			dest[2].y = point_set[id[j+2]].second.y;
			CvMat* warp_mat=cvCreateMat(2,3,CV_32FC1);
			cvGetAffineTransform(src,dest,warp_mat);
			int t_count = 0;
			for(int k = 0; k < l; k++){
				vector<Point3f> vec; 
				vec.push_back(Point3f(point_set[k].first.x,point_set[k].first.y,1)); 

				Mat srcMat = Mat(vec).reshape(1).t(); 
				Mat dstMat = Mat(warp_mat) * srcMat; //USE MATRIX ALGEBRA 

				dst=Point2f(dstMat.at<float>(0,0),dstMat.at<float>(1,0));
				if(e_dis(dst, Point2f(point_set[k].second)) < threshold)
					t_count ++;
			}
			r[x] = t_count;
		}
	}
	int max_i = 0, max_id = 0;
	for(int i = 0; i < count; i++)
		if(max_i < r[i]){
			max_i = r[i];
			max_id = i;
		}
	printf("found %d in %d\n", max_i, l);
	int id1 = max_id / (l * l), id2 = (max_id / l) % l, id3 = max_id % l;
	src[0].x = point_set[id1].first.x;
	src[0].y = point_set[id1].first.y;
	src[1].x = point_set[id2].first.x;
	src[1].y = point_set[id2].first.y;
	src[2].x = point_set[id3].first.x;
	src[2].y = point_set[id3].first.y;
	dest[0].x = point_set[id1].second.x;
	dest[0].y = point_set[id1].second.y;
	dest[1].x = point_set[id2].second.x;
	dest[1].y = point_set[id2].second.y;
	dest[2].x = point_set[id3].second.x;
	dest[2].y = point_set[id3].second.y;
	CvMat* warp_mat=cvCreateMat(2,3,CV_32FC1);
	cvGetAffineTransform(src,dest,warp_mat);
	for(int i = 0; i < l; i++){
		vector<Point3f> vec; 
		vec.push_back(Point3f(point_set[i].first.x,point_set[i].first.y,1)); 

		Mat srcMat = Mat(vec).reshape(1).t(); 
		Mat dstMat = Mat(warp_mat) * srcMat; //USE MATRIX ALGEBRA 

		dst=Point2f(dstMat.at<float>(0,0),dstMat.at<float>(1,0));
		if(e_dis(dst, Point2f(point_set[i].second)) < threshold)
			rsltId.push_back(i);
	}
}

void SIFTMatcher::clear(){
	if(processed){
		kdtree_release(root);
		processed = false;
	}
}
void SIFTMatcher::ransac(
	vector<pair<point, point>>& point_set,
	vector<vector<unsigned int>> flag, int block){

	vector<vector<pair<CvPoint, CvPoint>>> data;
	data.resize(block + 1);
	size_t l = point_set.size();
	for(int i = 0; i < l; i++){
		data[flag[point_set[i].first.y][point_set[i].first.x]].push_back(
			pair<CvPoint, CvPoint>(
			cvPoint(point_set[i].first.x, point_set[i].first.y),
			cvPoint(point_set[i].second.x, point_set[i].second.y)));
	}

	point_set.clear();
	for(int i = 0; i < block; i++){
		if(data[i].size() > 4){
			vector<int> id;
			RANSAC_pair(data[i], id);
			for(int j = 0; j < id.size(); j++){
				point_set.push_back(pair<point, point>(
					point(data[i][id[j]].first), point(data[i][id[j]].second))
					);
			}
		} else {
			for(int j = 0; j < data[i].size(); j++)
				point_set.push_back(pair<point, point>(point(data[i][j].first), point(data[i][j].second)));
		}
	}
}

int SIFTMatcher::getMatchedPointDuals(vector<pair<point, point>>& point_set, IplImage* left, IplImage* right){
	int n_l = sift_features(left, &feat_left);
	int n_r = sift_features(right, &feat_right);
	root = kdtree_build(feat_right, n_r);
	int ret = 0, k;
	double d0, d1;
	point pt_left, pt_right;
	point_set.clear();
	for(int i = 0; i < n_l; i++){
		t_feat = feat_left + i;
		k = kdtree_bbf_knn( root, t_feat, 2, &nbrs, KDTREE_BBF_MAX_NN_CHKS);
		if(k == 2){
			d0 = descr_dist_sq( t_feat, nbrs[0] );
			d1 = descr_dist_sq( t_feat, nbrs[1] );
			if(nbrs[0]->x - t_feat->x < 0 && d0 < d1 * NN_SQ_DIST_RATIO_THR && nbrs[0]->y - t_feat->y < 100)
			{
				pt_left = point( cvRound( t_feat->x ), cvRound( t_feat->y ) );
				pt_right = point( cvRound( nbrs[0]->x ), cvRound( nbrs[0]->y));
				point_set.push_back(pair<point, point>(pt_left, pt_right));
				ret++;
				feat_left[i].fwd_match = nbrs[0];
			}
		}
	}
	printf("match points : %d\n", point_set.size());
	processed = true;
	return ret;
}

SIFTMatcher::SIFTMatcher(){
	processed = false;
}

SIFTMatcher::~SIFTMatcher(){
	if(processed)
		clear();
}