#include "config.h"
#include "WatershedChopper.h"

void WatershedChopper::on_mouse(int event, int x, int y, int flags, void* param){
	WatershedChopper *chopper = reinterpret_cast<WatershedChopper*>(param);
	if( !chopper->img )
		return;
	if( event == CV_EVENT_RBUTTONUP || !(flags & CV_EVENT_FLAG_RBUTTON) ){
		if(chopper->pt_start.x > 0){
			chopper->pt_end = cvPoint(x, y);
			chopper->dir_arr.push_back(std::pair<CvPoint, CvPoint>(chopper->pt_start, chopper->pt_end));

			printf("[%d %d]-->[%d %d]: [%d %d]\n",
				chopper->pt_start.x,
				chopper->pt_start.y,
				chopper->pt_end.x,
				chopper->pt_end.y,
				chopper->pt_end.x - chopper->pt_start.x,
				chopper->pt_end.y - chopper->pt_start.y);
			chopper->pt_start = cvPoint(-1, -1);
		}
	}
	else if( event == CV_EVENT_MOUSEMOVE && (flags & CV_EVENT_FLAG_RBUTTON) )
	{
		chopper->pt_end = cvPoint(x, y);
	}
	else if(event == CV_EVENT_RBUTTONDOWN){
		chopper->pt_start = cvPoint(x, y);
	}
	
	if( event == CV_EVENT_LBUTTONUP || !(flags & CV_EVENT_FLAG_LBUTTON) )
		chopper->prev_pt = cvPoint(-1,-1);
	else if( event == CV_EVENT_LBUTTONDOWN )
		chopper->prev_pt = cvPoint(x,y);
	else if( event == CV_EVENT_MOUSEMOVE && (flags & CV_EVENT_FLAG_LBUTTON) )
	{
		CvPoint pt = cvPoint(x,y);
		if( chopper->prev_pt.x < 0 )
			chopper->prev_pt = pt;
		cvLine( chopper->marker_mask, chopper->prev_pt, pt, cvScalarAll(255), 5, 8, 0 );//CvScalar Աdouble val[4] RGBAֵA=alpha
		cvLine( chopper->img, chopper->prev_pt, pt, cvScalarAll(255), 5, 8, 0 );
		chopper->prev_pt = pt;
		cvShowImage( "image", chopper->img);
	}
}

WatershedChopper::WatershedChopper(){
	marker_mask = 0;
	markers = 0;
	img = 0;
	img0 = 0;
	img_gray = 0;
	wshed = 0;
	rng = cvRNG(-1);
	comp_count = 0;
	pt_start = cvPoint(-1, -1);
	prev_pt = cvPoint(-1, -1);
}

bool WatershedChopper::loadPicture(const char* filename){

	IplImage* t = cvLoadImage(filename);
	if(t == NULL)
		return false;
	double sc = 1080.0 / (double)t->width;
	img0 = cvCreateImage(cvSize(t->width * sc, t->height * sc), t->depth, t->nChannels);
	cvResize(t, img0);
	cvReleaseImage(&t);
	storage = cvCreateMemStorage(0);
	cvNamedWindow( "image", 1 );
	cvNamedWindow( "watershed transform", 1 );
	img = cvCloneImage( img0 );
	img_gray = cvCloneImage( img0 );
	wshed = cvCloneImage( img0 );
	dst = cvCloneImage( img0 );
	marker_mask = cvCreateImage( cvGetSize(img), 8, 1 );
	markers = cvCreateImage( cvGetSize(img), IPL_DEPTH_32S, 1 );
	cvCvtColor( img, marker_mask, CV_BGR2GRAY );
	cvCvtColor( marker_mask, img_gray, CV_GRAY2BGR );
	cvZero( marker_mask );
	cvZero( wshed );
	cvShowImage( "image", img );
	cvShowImage( "watershed transform", wshed );
	cvSetMouseCallback("image", &WatershedChopper::on_mouse, this);
	return true;
}

void WatershedChopper::refresh(){
	cvZero( marker_mask );
	cvCopy( img0, img );
	cvShowImage( "image", img );
	dir_arr.clear();
}

bool WatershedChopper::chop(){
	CvSeq* contours = 0;
	CvMat* color_tab = 0;
	int i, j;

	cvClearMemStorage(storage);
	cvFindContours( marker_mask, storage, &contours, sizeof(CvContour),
		CV_RETR_CCOMP, CV_CHAIN_APPROX_SIMPLE );
	cvZero( markers );
	for( ; contours != 0; contours = contours->h_next, comp_count++ )
	{
		cvDrawContours(markers, contours, cvScalarAll(comp_count+1),
			cvScalarAll(comp_count+1), -1, -1, 8, cvPoint(0,0) );
	}
	if( comp_count == 0 )
		return false;
	color_tab = cvCreateMat( 1, comp_count, CV_8UC3 );
	color_map.insert(std::pair<bgr_point, int>(bgr_point(255,255,255), comp_count + 1));
	for( i = 0; i < comp_count; i++ )
	{
		uchar* ptr = color_tab->data.ptr + i*3;
		ptr[0] = (uchar)(cvRandInt(&rng)%180 + 50);
		ptr[1] = (uchar)(cvRandInt(&rng)%180 + 50);
		ptr[2] = (uchar)(cvRandInt(&rng)%180 + 50);
		color_map.insert(std::pair<bgr_point, int>(bgr_point(ptr[0], ptr[1], ptr[2]), i + 1));
	}

	double t = (double)cvGetTickCount();
	cvWatershed( img0, markers );
	t = (double)cvGetTickCount() - t;
	printf( "exec time = %gms, %d components\n", t/(cvGetTickFrequency()*1000.), comp_count);

	for( i = 0; i < markers->height; i++ )
		for( j = 0; j < markers->width; j++ ){
			int idx = CV_IMAGE_ELEM( markers, int, i, j );
			uchar* dst = &CV_IMAGE_ELEM( wshed, uchar, i, j*3 );
			if( idx == -1 )
				dst[0] = dst[1] = dst[2] = (uchar)255;
			else if( idx <= 0 || idx > comp_count )
				dst[0] = dst[1] = dst[2] = (uchar)0;
			else
			{
				uchar* ptr = color_tab->data.ptr + (idx-1)*3;
				dst[0] = ptr[0]; dst[1] = ptr[1]; dst[2] = ptr[2];
			}
		}
	cvAddWeighted( wshed, 0.5, img_gray, 0.5, 0, dst );//wshed.x.y=0.5*wshed.x.y+0.5*img_gray+0Ȩںͼ
	cvShowImage( "watershed transform", dst );
	cvReleaseMat( &color_tab );
	return true;
}