#ifndef _DEFINE_H_
#define _DEFINE_H_

#include <cstdlib>
#include <cmath>
#include <algorithm>

#define MAX_QUEUE		1024

const double eps = 1e-6;

template<typename T>
class CMatrix {
public:
    CMatrix() : m_p(NULL), m_row(0), m_col(0) {}
    CMatrix(int n, int m) : m_p(NULL) { resize(n, m); }
    ~CMatrix() {
        if (m_p) delete[] m_p;
    }

    int row() const { return m_row; }
    int col() const { return m_col; }

    void resize(int n, int m) {
        if (m_p) delete[] m_p;
        m_p = new T[n * m];
        m_row = n; m_col = m;
    }

    T *operator [](int k) { return m_p + k * m_col; }

    void fill(T x) { std::fill(m_p, m_p + m_row * m_col, x); }

private:
	T *m_p;
    int m_row, m_col;
};

template<typename T>
class CQueue {
public:
	CQueue() : m_front(0), m_rear(0) {}
	~CQueue() {}

	int size() { return m_rear - m_front; }
	bool empty() { return m_front == m_rear; }

	void clear() { m_front = m_rear = 0; }
	void push(const T &x) { m_data[m_rear++] = x; }
	T pop() { return m_data[m_front++]; }
	T front() { return m_data[m_front]; }
	T back() { return m_data[rear - 1]; }

private:
	T m_data[MAX_QUEUE];
	int m_front, m_rear;
};

struct PixPoint {
	int x, y;

	PixPoint() : x(0), y(0) {}
	PixPoint(int _x, int _y) : x(_x), y(_y) {}

	PixPoint operator +(const PixPoint &p) const {
		return PixPoint(x + p.x, y + p.y);
	}
};

const PixPoint step[] = {
	PixPoint( 0, -1), PixPoint(-1, -1), PixPoint(-1,  0), PixPoint(-1,  1),
	PixPoint( 0,  1), PixPoint( 1,  1), PixPoint( 1,  0), PixPoint( 1, -1)
};

inline bool zero(double x) {
	return fabs(x) < eps;
}

inline double sq(double x) {
	return x * x;
}

inline double dis(const PixPoint &a, const PixPoint &b) {
	return sqrt(sq(a.x - b.x) + sq(a.y - b.y));
}

#endif